package top.ninwoo.edgecenter;

import org.junit.After;
import org.junit.Assert;
import org.junit.Before;
import org.junit.Test;
import org.junit.runner.RunWith;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.test.context.SpringBootTest;
import org.springframework.test.context.junit4.SpringRunner;
import top.ninwoo.common.entity.DockerContainer;
import top.ninwoo.utils.entity.ChainEntity;
import top.ninwoo.utils.entity.ChainType;
import top.ninwoo.utils.service.DockerService;
import top.ninwoo.utils.service.OVSService;
import top.ninwoo.utils.service.OvsDockerService;
import top.ninwoo.utils.service.TcService;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * 测试嵌入式Docker的服务接口
 * @Author joliu
 * @Description
 * @Date　Create in 下午8:41 2019/10/28
 */
@RunWith(SpringRunner.class)
@SpringBootTest
public class InDockerTests {
    private static final Logger LOG = LoggerFactory.getLogger(InDockerTests.class);
    @Autowired
    DockerService dockerService;

    @Autowired
    OVSService ovsService;

    @Autowired
    OvsDockerService ovsDockerService;

    @Autowired
    TcService tcService;

    DockerContainer dockerContainer1 = null;

    DockerContainer dockerContainer2 = null;

    private final String bridgeName = "testbr";
    /**
     * 构建两个docker
     */
    @Before
    public void init() {
        // 创建两个Docker
        dockerContainer1 = new DockerContainer();
        dockerContainer1.setName("RouterTestIn1");
        dockerContainer1.setImage("joliu/networktest");
        dockerContainer1.setCommand("iperf -s");
        dockerContainer1 = dockerService.runDocker(dockerContainer1);

        dockerContainer2 = new DockerContainer();
        dockerContainer2.setName("RouterTestIn2");
        dockerContainer2.setImage("joliu/networktest");
        dockerContainer2.setCommand("sh");
        dockerContainer2 = dockerService.runDocker(dockerContainer2);
        // 创建一个ovs网桥
        if(ovsService.isBridge(bridgeName)) {
            ovsService.delBridge(bridgeName);
        }
        ovsService.addBridge(bridgeName);

        try {
            Thread.sleep(500);
        } catch (InterruptedException e) {
            e.printStackTrace();
        }
        // 绑定两个Docker的网卡到ovs网桥上
        ovsDockerService.addPort(bridgeName, "eth1", dockerContainer1.getId(), "10.1.100.2/24");
        ovsDockerService.addPort(bridgeName, "eth1", dockerContainer2.getId(), "10.1.100.3/24");
    }

    @Test
    public void testInDocker() {
        String result = "";
        // 网络联通测试
        result = dockerService.execInDocker(dockerContainer1.getId(), new String[]{"ping", "10.1.100.3", "-c", "2"});
        System.out.println(result);
        // 使用Iptables进行端口限制
        result = dockerService.execInDocker(dockerContainer1.getId(), new String[]{"iptables", "-I", "INPUT", "-s", "10.1.100.3", "-j", "DROP"});
        System.out.println(result);
        // 测试网络
        result = dockerService.execInDocker(dockerContainer1.getId(), new String[]{"ping", "10.1.100.3", "-c", "2"});
        System.out.println(result);
    }

    @Test
    public void testQos() {
        String result = "";
        // 测试网络连通性
        result = dockerService.execInDocker(dockerContainer2.getId(), "ping 10.1.100.2 -c 2".split(" "));
        System.out.println(result);

        // 测试网络带宽
        result = dockerService.execInDocker(dockerContainer2.getId(), "iperf -c 10.1.100.2 -i -t 2".split(" "));
        System.out.println(result);

        // 开启限速
        System.out.println(dockerService.execInDocker(dockerContainer1.getId(), "tc qdisc add dev eth1 root handle 1: htb default 20"));
        // 下行总速率5Mbit
        System.out.println(dockerService.execInDocker(dockerContainer1.getId(), "tc class add dev eth1 parent 1:0 classid 1:1 htb rate 5Mbit"));
        // 1 号通道的下行速度，最小3Mbit 最大5 Mbit （还可以添加优先级）
        System.out.println(dockerService.execInDocker(dockerContainer1.getId(), "tc class add dev eth1 parent 1:1 classid 1:20 htb rate 4Mbit ceil 6Mbit"));
        // 使用了一个公平侧率
        System.out.println(dockerService.execInDocker(dockerContainer1.getId(), "tc qdisc add dev eth1 parent 1:20 handle 20: sfq perturb 10"));
        // 设置了一个过滤器
        System.out.println(dockerService.execInDocker(dockerContainer1.getId(), "tc filter add dev eth1 parent 1:20 protocol ip u32 match ip sport 5002 0xffff classid 1:20"));

        // 再次进行测速
        result = dockerService.execInDocker(dockerContainer2.getId(), "iperf -c 10.1.100.2 -i -t 2".split(" "));
        System.out.println(result);

        // 删除限速
        String res = dockerService.execInDocker(dockerContainer1.getId(), "tc qdisc del dev eth1 root");
        System.out.println(res);

        // 再次进行测速
        result = dockerService.execInDocker(dockerContainer2.getId(), "iperf -c 10.1.100.2 -i -t 2".split(" "));
        System.out.println(result);
    }

    @Test
    public void testTcDelay() {
        // 查看未加限制的网络情况
        LOG.info("未加任何限制的网络:\n" + pingTest());
        dockerService.execInDocker(dockerContainer1.getId(), "tc qdisc add dev eth1 root netem delay 200ms");
        LOG.info("添加200ms固定延迟的延迟：\n" + pingTest());
        dockerService.execInDocker(dockerContainer1.getId(), "tc qdisc change dev eth1 root netem delay 100ms 10ms");
        LOG.info("添加100ms延时, 并添加10ms的随机延迟" + pingTest());
        dockerService.execInDocker(dockerContainer1.getId(), "tc qdisc change dev eth1 root netem delay 100ms 10ms 25%");
        LOG.info("添加100ms延时, 并添加10ms的随机延迟, 25%相关性" + pingTest());
        dockerService.execInDocker(dockerContainer1.getId(), "tc qdisc change dev eth1 netem delay 100ms 20ms distribution normal");
        LOG.info("正态分布" + pingTest());
        dockerService.execInDocker(dockerContainer1.getId(), "tc qdisc change dev eth1 root netem loss 10%");
        LOG.info("10%丢包率：" + pingTest());

    }

    @Test
    public void testTcLoss() {
        // 查看未加限制的网络情况
        dockerService.execInDocker(dockerContainer1.getId(), "tc qdisc add dev eth1 root netem loss 10%");
        //tcService.addDelay(dockerContainer1.getId());
        LOG.info("10%丢包率：" + testLoss());

    }

    public String testLoss() {
        // 需要安装mtr
        String cmd = "mtr -r -c 30 -s 1024 10.1.100.2";
        return dockerService.execInDocker(dockerContainer2.getId(), cmd);
    }

    @Test
    public void testTcQos() {
        // 测试网络带宽
        LOG.info("为做任何限制的网络：\n" + basicQos());
        dockerService.execInDocker(dockerContainer1.getId(), "tc qdisc add dev eth1 root tbf rate 1mbit burst 32kbit latency 400ms");
        LOG.info("限制带宽为1mbit的网络：\n" + basicQos());


    }

    public String basicQos() {
        String result = dockerService.execInDocker(dockerContainer2.getId(), "iperf -c 10.1.100.2 -i -t 2".split(" "));
        return result;
    }



    public String pingTest() {
        return dockerService.execInDocker(dockerContainer1.getId(), "ping 10.1.100.3 -c 10");
    }

    @Test
    public void testParseIptablesDetail() {
        /**
         * Chain INPUT (policy ACCEPT 0 packets, 0 bytes)
         *  pkts bytes target     prot opt in     out     source               destination
         *     0     0 DROP       all  --  *      *       10.1.100.3           0.0.0.0/0
         *
         * Chain FORWARD (policy ACCEPT 0 packets, 0 bytes)
         *  pkts bytes target     prot opt in     out     source               destination
         *
         * Chain OUTPUT (policy ACCEPT 0 packets, 0 bytes)
         *  pkts bytes target     prot opt in     out     source               destination
         */
        Map<String, List<ChainEntity>> map = new HashMap<>();
        map.put("INPUT", new ArrayList<>());
        map.put("FORWARD", new ArrayList<>());
        map.put("OUTPUT", new ArrayList<>());
        String result = "";
        result = dockerService.execInDocker(dockerContainer1.getId(), new String[]{"iptables", "-I", "INPUT", "-s", "10.1.100.3", "-j", "DROP"});
        System.out.println(result);
        result = dockerService.execInDocker(dockerContainer1.getId(), new String[]{"iptables", "-vnL"});
        String[] lines = result.split("\n");
        ChainEntity.Builder builder = ChainEntity.builder();
        String key = "";
        for (int i = 0; i < lines.length; i++) {
            if(lines[i].startsWith("Chain INPUT")) {
                builder = ChainEntity.builder().type(ChainType.INPUT);
                key = "INPUT";
                i++;
                continue;
            } else if(lines[i].startsWith("Chain FORWARD")) {
                builder = ChainEntity.builder().type(ChainType.FORWARD);
                key = "FORWARD";
                i++;
                continue;
            } else if(lines[i].startsWith("Chain OUTPUT")) {
                builder = ChainEntity.builder().type(ChainType.OUTPUT);
                key = "OUTPUT";
                i++;
                continue;
            }

            if("".equals(lines[i])) {
                continue;
            }


            String[] s = lines[i].replaceAll(" +", " ").trim().split(" ");
            ChainEntity chain;
            if(s.length == 9) {
                chain = builder.pkts(Long.parseLong(s[0]))
                        .bytes(Long.parseLong(s[1]))
                        .target(s[2])
                        .prot(s[3])
                        .opt(s[4])
                        .in(s[5])
                        .out(s[6])
                        .source(s[7])
                        .destination(s[8])
                        .build();
                map.get(key).add(chain);
            }

        }
        System.out.println(result);
    }

    /**
     * 这里主要测试接口限速，以及统计实时流量，需要外部辅助执行其他命令
     */
    @Test
    public void testTCService() throws InterruptedException {
        // 限速
        //System.out.println(tcService.addQos(dockerContainer2.getId(), "4mbit", "400ms"));
        // 测试丢包率
        System.out.println(tcService.addDelay(dockerContainer2.getId(), 10));
        //dockerService.execInDocker(dockerContainer2.getId(), "tc qdisc add dev eth1 root tbf rate 1mbit limit 10k burst 10k mtu 5000");
        //dockerService.execInDocker(dockerContainer1.getId(), "tc qdisc add dev eth1 root handle 1: htb default 12");
        //dockerService.execInDocker(dockerContainer1.getId(), "tc class add dev eth1 parent 1: classid 1:12 htb rate 1kbit");

        Thread thread = new Thread(new Runnable() {
            @Override
            public void run() {
                while(!Thread.currentThread().isInterrupted()) {
                    try {
                        System.out.println("Network Usage:" + tcService.networkUsage(dockerContainer2.getId()));
                    } catch (InterruptedException e) {
                        Thread.currentThread().interrupt();
                    }
                }
            }
        });

        thread.start();
        Thread.sleep(1000);
        System.out.println(pingTest());
        String result = dockerService.execInDocker(dockerContainer2.getId(), "iperf -c 10.1.100.2 -i -t 10".split(" "));
        thread.interrupt();

        System.out.println(result);

    }

    @After
    public void close() {
        // 清除Docker
        Assert.assertTrue(dockerService.deleteDockerById("RouterTestIn1"));
        Assert.assertTrue(dockerService.deleteDockerById("RouterTestIn2"));
        Assert.assertTrue(ovsService.delBridge("testbr"));
    }
}
